"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;
var _fs = _interopRequireDefault(require("fs"));
var _uuid = require("uuid");
var _http = _interopRequireDefault(require("http"));
var _https = _interopRequireDefault(require("https"));
var _express = _interopRequireDefault(require("express"));
var _bodyParser = _interopRequireDefault(require("body-parser"));
var _cors = _interopRequireDefault(require("cors"));
var _geoipLite = _interopRequireDefault(require("geoip-lite"));
var _useragent = _interopRequireDefault(require("useragent"));
var _ws = _interopRequireDefault(require("ws"));
var _configYml = _interopRequireDefault(require("config-yml"));
var _configureStore = _interopRequireDefault(require("./store/configureStore"));
var appInfo = _interopRequireWildcard(require("./actions/appInfo"));
var wsBroker = _interopRequireWildcard(require("./actions/wsBroker"));
var _zmqBroker = require("./selectors/zmqBroker");
function _getRequireWildcardCache(e) { if ("function" != typeof WeakMap) return null; var r = new WeakMap(), t = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(e) { return e ? t : r; })(e); }
function _interopRequireWildcard(e, r) { if (!r && e && e.__esModule) return e; if (null === e || "object" != _typeof(e) && "function" != typeof e) return { "default": e }; var t = _getRequireWildcardCache(r); if (t && t.has(e)) return t.get(e); var n = { __proto__: null }, a = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var u in e) if ("default" !== u && {}.hasOwnProperty.call(e, u)) { var i = a ? Object.getOwnPropertyDescriptor(e, u) : null; i && (i.get || i.set) ? Object.defineProperty(n, u, i) : n[u] = e[u]; } return n["default"] = e, t && t.set(e, n), n; }
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }
// import uuid from 'uuid';

var app = (0, _express["default"])();
app.use(_bodyParser["default"].json());
app.use(_bodyParser["default"].urlencoded({
  extended: false
}));
app.use((0, _cors["default"])({
  allowedOrigins: '*'
}));
app.disable('x-powered-by');
if (_configYml["default"].zmq2ws.ssl.enabled) {
  var privateKey = _fs["default"].readFileSync(_configYml["default"].zmq2ws.ssl.key, 'utf8');
  var certificate = _fs["default"].readFileSync(_configYml["default"].zmq2ws.ssl.cert, 'utf8');
  var credentials = {
    key: privateKey,
    cert: certificate
  };
  app.server = _https["default"].createServer(credentials, app);
  app.set('port', _configYml["default"].zmq2ws.port);
} else {
  app.server = _http["default"].createServer(app);
  app.set('port', _configYml["default"].zmq2ws.port);
}
app.wss = new _ws["default"].Server({
  server: app.server
});
app.wss.broadcast = function broadcast(data) {
  app.wss.clients.forEach(function each(client) {
    if (client.readyState === _ws["default"].OPEN) {
      client.send(data);
    }
  });
};
app.wss.on('connection', function (ws, req) {
  // generate id of websocket
  ws.id = (0, _uuid.v4)();

  // get number of clients
  var nclients = Array.from(app.wss.clients).length;
  console.log('[' + nclients + '] User ' + ws.id + ' was connected');
  var ip = req.headers['x-real-ip'] || req.connection.remoteAddress || null;
  var geo = ip ? _geoipLite["default"].lookup(ip) : null;
  // get ip and user info
  var userInfo = {
    id: ws.id,
    ip: ip,
    geo: geo,
    info: _useragent["default"].lookup(req.headers['user-agent']).toJSON(),
    username: ""
  };
  _configureStore["default"].dispatch(appInfo.onWebSocketEnter(app.wss, userInfo));
  var wsInfo = {
    type: "ws",
    payload: {
      id: ws.id
    }
  };
  ws.send(JSON.stringify(wsInfo));
  ws.on('message', function (msg) {
    console.log('received: %s', msg);
    var o = JSON.parse(msg);
    var n = "";
    if (o.src === undefined) o.src = "obmon";
    if (o.type === "sub") {
      var b = (0, _zmqBroker.getZeroMQBroker)(_configureStore["default"].getState()).zmqs$;
      n = o.src + "|" + o.name + "|" + o.sub;
      if (b[n] !== undefined) {
        if (b[n].zmq$ !== undefined) {
          _configureStore["default"].dispatch(wsBroker.onWebSocketSub(ws, b[n].zmq$, n));
        }
      } else {
        console.log("Ws:onMesage => We could not find service : " + n);
      }
    } else if (o.type === "unsub") {
      n = o.src + "|" + o.name + "|" + o.sub;
      _configureStore["default"].dispatch(wsBroker.onWebSocketUnSub(ws, n));
    } else if (o.type === "updateinfo") {
      console.log("Info update for ws " + ws.id + " username: " + o.username + " uid: " + o.uid);
      _configureStore["default"].dispatch(appInfo.onWebSocketUserLogin(app.wss, ws.id, o.username, o.username));
    } else {
      // ws.send(JSON.stringify({ type: "zmq", src: "" }));
    }
  });
  ws.on('close', function () {
    _configureStore["default"].dispatch(appInfo.onWebSocketExit(app.wss, {
      id: ws.id
    }));
    var nclients = Array.from(app.wss.clients).length;
    console.log('[' + nclients + '] User ' + ws.id + ' was disconnected');
    _configureStore["default"].dispatch(wsBroker.onWebSocketUnSub(ws, ""));
  });
  ws.on('error', function () {
    return console.log('Error from web socket');
  });
});
app.get('*', function (req, res) {
  res.sendFile(__dirname + '/index.html');
});
var _default = exports["default"] = app;